﻿using System;
using System.ComponentModel;
using Wisej.Web;
using Microsoft.Reporting.WebForms;
using Wisej.Web.Ext.AspNetControl;
using System.Web.UI;
using Wisej.Core;
using System.Runtime.CompilerServices;

namespace Wisej.AspNetReportViewer
{
	/// <summary>
	/// Sample ReportViewer wrapper.
	/// </summary>
	public class AspNetReportViewer : AspNetWrapper<ReportViewer>
	{
		/// <summary>
		/// Returns or sets the URL of the Reporting Services server.
		/// </summary>
		public string ServerUrl
		{
			get { return this._serverUrl; }

			set
			{
				value = value ?? "";

				if (this._serverUrl != value)
				{
					this._serverUrl = value;
				}
			}
		}
		private string _serverUrl = "";

		/// <summary>
		/// Returns or sets the path to the report on the server.
		/// </summary>
		[DefaultValue("")]
		public string ReportPath
		{
			get { return this._reportPath; }

			set
			{
				value = value ?? "";

				if (this._reportPath != value)
				{
					this._reportPath = value;
				}
			}
		}
		private string _reportPath = "";

		/// <summary>
		/// Returns or sets the authentication user name.
		/// </summary>
		[DefaultValue(null)]
		public string UserName
		{
			get;
			set;
		}

		/// <summary>
		/// Returns or sets the authentication password.
		/// </summary>
		[DefaultValue(null)]
		public string Password
		{
			get;
			set;
		}

		/// <summary>
		/// Returns or sets the authentication domain name.
		/// </summary>
		[DefaultValue(null)]
		public string DomainName
		{
			get;
			set;
		}

		/// <summary>
		/// Returns the value of the IsLoading property.
		/// </summary>
		/// <returns>True when the report view control is still loading data and rendering.</returns>
		public bool IsLoading
		{
			get;
			private set;
		}

		protected override void OnInit(EventArgs e)
		{
			var viewer = this.WrappedControl;
			viewer.ServerReport.ReportPath = this.ReportPath;
			viewer.ServerReport.ReportServerUrl = new Uri(this.ServerUrl);
			viewer.ProcessingMode = Microsoft.Reporting.WebForms.ProcessingMode.Remote;

			if (!this.IsPostBack && this.UserName != null)
				viewer.ServerReport.ReportServerCredentials = new ReportViewerCredentials(this.UserName, this.Password, this.DomainName);

			// attach the handled events to fire the event on the wrapper.
			if (this.Toggle != null)
				viewer.Toggle += viewer_Toggle;
			if (this.BookmarkNavigation != null)
				viewer.BookmarkNavigation += viewer_BookmarkNavigation;
			if (this.PageNavigation != null)
				viewer.PageNavigation += viewer_PageNavigation;

			if (!this.IsPostBack && this.IsLoadingChanged != null)
			{
				string script = String.Format(
				@"Sys.Application.add_load(function() {{
					$find('{1}').add_propertyChanged(function(sender, e) {{
						if (e.get_propertyName()=='isLoading')
							window.top.Wisej.Core.getComponent('id_{0}').fireDataEvent('loadingChanged', sender.get_isLoading());
					}});
				}});
				", this.Handle, viewer.ID);

				ScriptManager.RegisterStartupScript(this.Page, typeof(System.Web.UI.Page), "isLoadingCallback", script, true);
			}

			base.OnInit(e);
		}

		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);
		}

		void viewer_BookmarkNavigation(object sender, BookmarkNavigationEventArgs e)
		{
			// this call is coming from the AspNet handler. if the
			// event handler in the wisej application updates a control, we need
			// either call Application.Update, or run the handler in an Update block to
			// push any update back to the client after the block is executed.
			Application.Update(this, () =>
			{
				BookmarkNavigation(sender, e);
			});
		}

		void viewer_PageNavigation(object sender, PageNavigationEventArgs e)
		{
			Application.Update(this, () =>
			{
				PageNavigation(sender, e);
			});
		}

		void viewer_Toggle(object sender, CancelEventArgs e)
		{
			Application.Update(this, () =>
			{
				Toggle(sender, e);
			});
		}

		/// <summary>
		/// Toggle event.
		/// </summary>
		public event CancelEventHandler Toggle;

		/// <summary>
		/// PageNavigation event.
		/// </summary>
		public event PageNavigationEventHandler PageNavigation;

		/// <summary>
		/// BookmarkNavigation event.
		/// </summary>
		public event BookmarkNavigationEventHandler BookmarkNavigation;

		/// <summary>
		/// IsLoadingChanged event.
		/// </summary>
		public event EventHandler IsLoadingChanged;

		/// <summary>
		/// Processes the event from the client.
		/// </summary>
		/// <param name="e"></param>
		protected override void OnWebEvent(WisejEventArgs e)
		{
			switch (e.Type)
			{
				case"loadingChanged":
					this.IsLoading = e.Parameters.Value ?? false;
					IsLoadingChanged(this, new PropertyChangedEventArgs(""));
					break;

				default:
					base.OnWebEvent(e);
					break;
			}
		}

		/// <summary>
		/// Renders the client component.
		/// </summary>
		/// <param name="config"></param>
		protected override void OnWebRender(dynamic config)
		{
			base.OnWebRender((object)config);

			config.wiredEvents.Add("loadingChanged(Value)");
		}
	}
}